@@ -115,8 +115,8 @@ class TestCaseWithKeyManager(BaseLeapTest):
             'username': address,
             'password': '<password>',
             'encrypted_only': True,
-            'cert': 'src/leap/mail/smtp/tests/cert/server.crt',
-            'key': 'src/leap/mail/smtp/tests/cert/server.key',
+            'cert': u'src/leap/mail/smtp/tests/cert/server.crt',
+            'key': u'src/leap/mail/smtp/tests/cert/server.key',
         }
 
         class Response(object):
